@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -58,9 +58,10 @@ private:
 
 
 ThumbnailBase::ThumbnailBase(
-	ThumbnailPixmapCache& thumbnail_cache, QSizeF const& max_size,
-	ImageId const& image_id, ImageTransformation const& image_xform)
-:	m_rThumbnailCache(thumbnail_cache),
+	IntrusivePtr<ThumbnailPixmapCache> const& thumbnail_cache,
+	QSizeF const& max_size, ImageId const& image_id,
+	ImageTransformation const& image_xform)
+:	m_ptrThumbnailCache(thumbnail_cache),
 	m_maxSize(max_size),
 	m_imageId(image_id),
 	m_imageXform(image_xform),
@@ -90,7 +91,7 @@ ThumbnailBase::paint(QPainter* painter,
 			new LoadCompletionHandler(this)
 		);
 		ThumbnailPixmapCache::Status const status =
-			m_rThumbnailCache.loadRequest(m_imageId, pixmap, handler);
+			m_ptrThumbnailCache->loadRequest(m_imageId, pixmap, handler);
 		if (status == ThumbnailPixmapCache::QUEUED) {
 			m_ptrCompletionHandler.swap(handler);
 		}
@@ -226,7 +227,9 @@ void
 ThumbnailBase::setImageXform(ImageTransformation const& image_xform)
 {
 	m_imageXform = image_xform;
-	QSizeF const unscaled_size(image_xform.resultingRect().size());
+	QSizeF const unscaled_size(
+		image_xform.resultingRect().size().expandedTo(QSizeF(1, 1))
+	);
 	QSizeF scaled_size(unscaled_size);
 	scaled_size.scale(m_maxSize, Qt::KeepAspectRatio);
 	
