@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -52,10 +52,10 @@ Filter::getName() const
 	return QCoreApplication::translate("deskew::Filter", "Deskew");
 }
 
-PageSequence::View
+PageView
 Filter::getView() const
 {
-	return PageSequence::PAGE_VIEW;
+	return PAGE_VIEW;
 }
 
 void
