@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,12 +39,12 @@
 namespace output
 {
 
-Filter::Filter(IntrusivePtr<PageSequence> const& pages,
+Filter::Filter(
 	PageSelectionAccessor const& page_selection_accessor)
 :	m_ptrSettings(new Settings)
 {
 	m_ptrOptionsWidget.reset(
-		new OptionsWidget(m_ptrSettings, pages, page_selection_accessor)
+		new OptionsWidget(m_ptrSettings, page_selection_accessor)
 	);
 }
 
@@ -58,10 +58,10 @@ Filter::getName() const
 	return QCoreApplication::translate("output::Filter", "Output");
 }
 
-PageSequence::View
+PageView
 Filter::getView() const
 {
-	return PageSequence::PAGE_VIEW;
+	return PAGE_VIEW;
 }
 
 void
@@ -99,7 +99,8 @@ Filter::writePageSettings(
 	QDomElement page_el(doc.createElement("page"));
 	page_el.setAttribute("id", numeric_id);
 
-	page_el.appendChild(m_ptrSettings->zonesForPage(page_id).toXml(doc, "zones"));
+	page_el.appendChild(m_ptrSettings->pictureZonesForPage(page_id).toXml(doc, "zones"));
+	page_el.appendChild(m_ptrSettings->fillZonesForPage(page_id).toXml(doc, "fill-zones"));
 	page_el.appendChild(params.toXml(doc, "params"));
 	
 	std::auto_ptr<OutputParams> output_params(m_ptrSettings->getOutputParams(page_id));
@@ -141,9 +142,14 @@ Filter::loadSettings(ProjectReader const& reader, QDomElement const& filters_el)
 			continue;
 		}
 		
-		ZoneSet const zones(el.namedItem("zones").toElement(), m_zonePropFactory);
-		if (!zones.empty()) {
-			m_ptrSettings->setZones(page_id, zones);
+		ZoneSet const picture_zones(el.namedItem("zones").toElement(), m_pictureZonePropFactory);
+		if (!picture_zones.empty()) {
+			m_ptrSettings->setPictureZones(page_id, picture_zones);
+		}
+
+		ZoneSet const fill_zones(el.namedItem("fill-zones").toElement(), m_fillZonePropFactory);
+		if (!fill_zones.empty()) {
+			m_ptrSettings->setFillZones(page_id, fill_zones);
 		}
 
 		QDomElement const params_el(el.namedItem("params").toElement());
@@ -162,24 +168,25 @@ Filter::loadSettings(ProjectReader const& reader, QDomElement const& filters_el)
 
 IntrusivePtr<Task>
 Filter::createTask(
-	PageId const& page_id, int const page_num,
-	QString const& out_dir, ThumbnailPixmapCache& thumbnail_cache,
+	PageId const& page_id,
+	IntrusivePtr<ThumbnailPixmapCache> const& thumbnail_cache,
+	OutputFileNameGenerator const& out_file_name_gen,
 	bool const batch, bool const debug)
 {
 	return IntrusivePtr<Task>(
 		new Task(
 			IntrusivePtr<Filter>(this), m_ptrSettings,
-			thumbnail_cache, page_id, page_num, out_dir,
-			batch, debug
+			thumbnail_cache, page_id, out_file_name_gen,
+			m_ptrOptionsWidget->lastTab(), batch, debug
 		)
 	);
 }
 
 IntrusivePtr<CacheDrivenTask>
-Filter::createCacheDrivenTask(QString const& out_dir)
+Filter::createCacheDrivenTask(OutputFileNameGenerator const& out_file_name_gen)
 {
 	return IntrusivePtr<CacheDrivenTask>(
-		new CacheDrivenTask(m_ptrSettings, out_dir)
+		new CacheDrivenTask(m_ptrSettings, out_file_name_gen)
 	);
 }
 
