@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,14 +16,19 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "OptionsWidget.h"
 #include "OptionsWidget.h.moc"
 #include "ChangeDpiDialog.h"
 #include "ApplyColorsDialog.h"
 #include "Settings.h"
+#include "Params.h"
+#include "DespeckleLevel.h"
 #include "ZoneSet.h"
 #include "PictureZoneComparator.h"
+#include "FillZoneComparator.h"
 #include "../../Utils.h"
 #include "ScopedIncDec.h"
+#include "config.h"
 #include <boost/foreach.hpp>
 #include <QVariant>
 #include <QColorDialog>
@@ -38,15 +43,20 @@
 namespace output
 {
 
-OptionsWidget::OptionsWidget(IntrusivePtr<Settings> const& settings,
-	IntrusivePtr<PageSequence> const& pages,
+OptionsWidget::OptionsWidget(
+	IntrusivePtr<Settings> const& settings,
 	PageSelectionAccessor const& page_selection_accessor)
 :	m_ptrSettings(settings),
-	m_ptrPages(pages),
 	m_pageSelectionAccessor(page_selection_accessor),
+	m_despeckleLevel(DESPECKLE_NORMAL),
+	m_lastTab(TAB_OUTPUT),
 	m_ignoreThresholdChanges(0)
 {
 	setupUi(this);
+
+#if !defined(ENABLE_DEWARPING)
+	dewarpPanel->hide();
+#endif
 	
 	colorModeSelector->addItem(tr("Black and White"), ColorParams::BLACK_AND_WHITE);
 	colorModeSelector->addItem(tr("Color / Grayscale"), ColorParams::COLOR_GRAYSCALE);
@@ -79,8 +89,8 @@ OptionsWidget::OptionsWidget(IntrusivePtr<Settings> const& settings,
 		this, SLOT(equalizeIlluminationToggled(bool))
 	);
 	connect(
-		despeckleCB, SIGNAL(clicked(bool)),
-		this, SLOT(despeckleToggled(bool))
+		dewarpCB, SIGNAL(clicked(bool)),
+		this, SLOT(dewarpToggled(bool))
 	);
 	connect(
 		lighterThresholdLink, SIGNAL(linkActivated(QString const&)),
@@ -96,12 +106,37 @@ OptionsWidget::OptionsWidget(IntrusivePtr<Settings> const& settings,
 	);
 	connect(
 		thresholdSlider, SIGNAL(valueChanged(int)),
-		this, SLOT(bwThresholdChanged(int))
+		this, SLOT(bwThresholdChanged())
+	);
+	connect(
+		thresholdSlider, SIGNAL(sliderReleased()),
+		this, SLOT(bwThresholdChanged())
 	);
 	connect(
 		applyColorsButton, SIGNAL(clicked()),
 		this, SLOT(applyColorsButtonClicked())
 	);
+
+	connect(
+		despeckleOffBtn, SIGNAL(clicked()),
+		this, SLOT(despeckleOffSelected())
+	);
+	connect(
+		despeckleCautiousBtn, SIGNAL(clicked()),
+		this, SLOT(despeckleCautiousSelected())
+	);
+	connect(
+		despeckleNormalBtn, SIGNAL(clicked()),
+		this, SLOT(despeckleNormalSelected())
+	);
+	connect(
+		despeckleAggressiveBtn, SIGNAL(clicked()),
+		this, SLOT(despeckleAggressiveSelected())
+	);
+	connect(
+		applyDespeckleButton, SIGNAL(clicked()),
+		this, SLOT(applyDespeckleButtonClicked())
+	);
 }
 
 OptionsWidget::~OptionsWidget()
@@ -111,9 +146,11 @@ OptionsWidget::~OptionsWidget()
 void
 OptionsWidget::preUpdateUI(PageId const& page_id)
 {
+	Params const params(m_ptrSettings->getParams(page_id));
 	m_pageId = page_id;
-	m_dpi = m_ptrSettings->getDpi(page_id);
-	m_colorParams = m_ptrSettings->getColorParams(page_id);
+	m_outputDpi = params.outputDpi();
+	m_colorParams = params.colorParams();
+	m_despeckleLevel = params.despeckleLevel();
 	updateDpiDisplay();
 	updateColorsDisplay();
 }
@@ -124,17 +161,10 @@ OptionsWidget::postUpdateUI()
 }
 
 void
-OptionsWidget::reloadIfZonesChanged()
+OptionsWidget::tabChanged(ImageViewTab const tab)
 {
-	ZoneSet saved_zones;
-	std::auto_ptr<OutputParams> output_params(m_ptrSettings->getOutputParams(m_pageId));
-	if (output_params.get()) {
-		saved_zones = output_params->zones();
-	}
-
-	if (!PictureZoneComparator::equal(saved_zones, m_ptrSettings->zonesForPage(m_pageId))) {
-		emit reloadRequested();
-	}
+	m_lastTab = tab;
+	reloadIfNecessary();
 }
 
 void
@@ -173,10 +203,10 @@ OptionsWidget::equalizeIlluminationToggled(bool const checked)
 }
 
 void
-OptionsWidget::despeckleToggled(bool const checked)
+OptionsWidget::dewarpToggled(bool const checked)
 {
 	BlackWhiteOptions opt(m_colorParams.blackWhiteOptions());
-	opt.setDespeckle(checked);
+	opt.setDewarp(checked);
 	m_colorParams.setBlackWhiteOptions(opt);
 	m_ptrSettings->setColorParams(m_pageId, m_colorParams);
 	emit reloadRequested();
@@ -201,8 +231,9 @@ OptionsWidget::setNeutralThreshold()
 }
 
 void
-OptionsWidget::bwThresholdChanged(int const value)
+OptionsWidget::bwThresholdChanged()
 {
+	int const value = thresholdSlider->value();
 	QString const tooltip_text(QString::number(value));
 	thresholdSlider->setToolTip(tooltip_text);
 	
@@ -222,7 +253,19 @@ OptionsWidget::bwThresholdChanged(int const value)
 	tooltip_pos = thresholdSlider->mapToGlobal(tooltip_pos);
 	QToolTip::showText(tooltip_pos, tooltip_text, thresholdSlider);
 	
+	if (thresholdSlider->isSliderDown()) {
+		// Wait for it to be released.
+		// We could have just disabled tracking, but in that case we wouldn't
+		// be able to show tooltips with a precise value.
+		return;
+	}
+
 	BlackWhiteOptions opt(m_colorParams.blackWhiteOptions());
+	if (opt.thresholdAdjustment() == value) {
+		// Didn't change.
+		return;
+	}
+
 	opt.setThresholdAdjustment(value);
 	m_colorParams.setBlackWhiteOptions(opt);
 	m_ptrSettings->setColorParams(m_pageId, m_colorParams);
@@ -233,13 +276,17 @@ void
 OptionsWidget::changeDpiButtonClicked()
 {
 	ChangeDpiDialog* dialog = new ChangeDpiDialog(
-		this, m_dpi, m_ptrPages, m_pageSelectionAccessor
+		this, m_outputDpi, m_pageId, m_pageSelectionAccessor
 	);
 	dialog->setAttribute(Qt::WA_DeleteOnClose);
 	connect(
 		dialog, SIGNAL(accepted(std::set<PageId> const&, Dpi const&)),
 		this, SLOT(dpiChanged(std::set<PageId> const&, Dpi const&))
 	);
+	connect(
+		dialog, SIGNAL(acceptedForAllPages(Dpi const&)),
+		this, SLOT(dpiChangedForAllPages(Dpi const&))
+	);
 	dialog->show();
 }
 
@@ -247,59 +294,183 @@ void
 OptionsWidget::applyColorsButtonClicked()
 {
 	ApplyColorsDialog* dialog = new ApplyColorsDialog(
-		this, m_ptrPages, m_pageSelectionAccessor
+		this, m_pageId, m_pageSelectionAccessor
 	);
 	dialog->setAttribute(Qt::WA_DeleteOnClose);
 	connect(
 		dialog, SIGNAL(accepted(std::set<PageId> const&)),
 		this, SLOT(applyColorsConfirmed(std::set<PageId> const&))
 	);
+	connect(
+		dialog, SIGNAL(acceptedForAllPages()),
+		this, SLOT(applyColorsToAllPagesConfirmed())
+	);
 	dialog->show();
 }
 
 void
 OptionsWidget::dpiChanged(std::set<PageId> const& pages, Dpi const& dpi)
 {
-	m_dpi = dpi;
 	updateDpiDisplay();
 	
-	if (int(pages.size()) == m_ptrPages->numImages()) {
-		m_ptrSettings->setDpiForAllPages(dpi);
-		emit invalidateAllThumbnails();
-	} else {
-		BOOST_FOREACH(PageId const& page_id, pages) {
-			m_ptrSettings->setDpi(page_id, dpi);
-		}
+	BOOST_FOREACH(PageId const& page_id, pages) {
+		m_ptrSettings->setDpi(page_id, dpi);
+		emit invalidateThumbnail(page_id);
 	}
 	
+	if (pages.find(m_pageId) != pages.end()) {
+		m_outputDpi = dpi;
+		emit reloadRequested();
+	}
+}
+
+void
+OptionsWidget::dpiChangedForAllPages(Dpi const& dpi)
+{
+	m_outputDpi = dpi;
+	updateDpiDisplay();
+
+	m_ptrSettings->setDpiForAllPages(dpi);
+	emit invalidateAllThumbnails();
 	emit reloadRequested();
 }
 
 void
 OptionsWidget::applyColorsConfirmed(std::set<PageId> const& pages)
 {
-	if (int(pages.size()) == m_ptrPages->numImages()) {
-		m_ptrSettings->setColorParamsForAllPages(m_colorParams);
-		emit invalidateAllThumbnails();
-	} else {
-		BOOST_FOREACH(PageId const& page_id, pages) {
-			m_ptrSettings->setColorParams(page_id, m_colorParams);
-		}
+	BOOST_FOREACH(PageId const& page_id, pages) {
+		m_ptrSettings->setColorParams(page_id, m_colorParams);
+		emit invalidateThumbnail(page_id);
 	}
 	
+	if (pages.find(m_pageId) != pages.end()) {
+		emit reloadRequested();
+	}
+}
+
+void
+OptionsWidget::applyColorsToAllPagesConfirmed()
+{
+	m_ptrSettings->setColorParamsForAllPages(m_colorParams);
+	emit invalidateAllThumbnails();
+
 	emit reloadRequested();
 }
 
+void
+OptionsWidget::despeckleOffSelected()
+{
+	handleDespeckleLevelChange(DESPECKLE_OFF);
+}
+
+void
+OptionsWidget::despeckleCautiousSelected()
+{
+	handleDespeckleLevelChange(DESPECKLE_CAUTIOUS);
+}
+
+void
+OptionsWidget::despeckleNormalSelected()
+{
+	handleDespeckleLevelChange(DESPECKLE_NORMAL);
+}
+
+void
+OptionsWidget::despeckleAggressiveSelected()
+{
+	handleDespeckleLevelChange(DESPECKLE_AGGRESSIVE);
+}
+
+void
+OptionsWidget::handleDespeckleLevelChange(DespeckleLevel const level)
+{
+	m_despeckleLevel = level;
+	m_ptrSettings->setDespeckleLevel(m_pageId, level);
+
+	bool handled = false;
+	emit despeckleLevelChanged(level, &handled);
+	
+	if (handled) {
+		// This means we are on the "Despeckling" tab.
+		emit invalidateThumbnail(m_pageId);
+	} else {
+		emit reloadRequested();
+	}
+}
+
+void
+OptionsWidget::applyDespeckleButtonClicked()
+{
+	ApplyColorsDialog* dialog = new ApplyColorsDialog(
+		this, m_pageId, m_pageSelectionAccessor
+	);
+	dialog->setAttribute(Qt::WA_DeleteOnClose);
+	dialog->setWindowTitle(tr("Apply Despeckling Level"));
+	connect(
+		dialog, SIGNAL(accepted(std::set<PageId> const&)),
+		this, SLOT(applyDespeckleConfirmed(std::set<PageId> const&))
+	);
+	connect(
+		dialog, SIGNAL(acceptedForAllPages()),
+		this, SLOT(applyDespeckleToAllPagesConfirmed())
+	);
+	dialog->show();
+}
+
+void
+OptionsWidget::applyDespeckleConfirmed(std::set<PageId> const& pages)
+{
+	BOOST_FOREACH(PageId const& page_id, pages) {
+		m_ptrSettings->setDespeckleLevel(page_id, m_despeckleLevel);
+		emit invalidateThumbnail(page_id);
+	}
+	
+	if (pages.find(m_pageId) != pages.end()) {
+		emit reloadRequested();
+	}
+}
+
+void
+OptionsWidget::applyDespeckleToAllPagesConfirmed()
+{
+	m_ptrSettings->setDespeckleLevelForAllPages(m_despeckleLevel);
+	emit invalidateAllThumbnails();
+	emit reloadRequested();
+}
+
+void
+OptionsWidget::reloadIfNecessary()
+{
+	ZoneSet saved_picture_zones;
+	ZoneSet saved_fill_zones;
+	DespeckleLevel saved_despeckle_level = DESPECKLE_NORMAL;
+	
+	std::auto_ptr<OutputParams> output_params(m_ptrSettings->getOutputParams(m_pageId));
+	if (output_params.get()) {
+		saved_picture_zones = output_params->pictureZones();
+		saved_fill_zones = output_params->fillZones();
+		saved_despeckle_level = output_params->outputImageParams().despeckleLevel();
+	}
+
+	if (!PictureZoneComparator::equal(saved_picture_zones, m_ptrSettings->pictureZonesForPage(m_pageId))) {
+		emit reloadRequested();
+	} else if (!FillZoneComparator::equal(saved_fill_zones, m_ptrSettings->fillZonesForPage(m_pageId))) {
+		emit reloadRequested();
+	} else if (saved_despeckle_level != m_ptrSettings->getDespeckleLevel(m_pageId)) {
+		emit reloadRequested();
+	}
+}
+
 void
 OptionsWidget::updateDpiDisplay()
 {
-	if (m_dpi.horizontal() != m_dpi.vertical()) {
+	if (m_outputDpi.horizontal() != m_outputDpi.vertical()) {
 		dpiLabel->setText(
 			QString::fromAscii("%1 x %2")
-			.arg(m_dpi.horizontal()).arg(m_dpi.vertical())
+			.arg(m_outputDpi.horizontal()).arg(m_outputDpi.vertical())
 		);
 	} else {
-		dpiLabel->setText(QString::number(m_dpi.horizontal()));
+		dpiLabel->setText(QString::number(m_outputDpi.horizontal()));
 	}
 }
 
@@ -337,10 +508,27 @@ OptionsWidget::updateColorsDisplay()
 	}
 	
 	bwOptions->setVisible(bw_options_visible);
+	despecklePanel->setVisible(bw_options_visible);
 	if (bw_options_visible) {
-		despeckleCB->setChecked(
-			m_colorParams.blackWhiteOptions().despeckle()
+		dewarpCB->setChecked(
+			m_colorParams.blackWhiteOptions().dewarp()
 		);
+
+		switch (m_despeckleLevel) {
+			case DESPECKLE_OFF:
+				despeckleOffBtn->setChecked(true);
+				break;
+			case DESPECKLE_CAUTIOUS:
+				despeckleCautiousBtn->setChecked(true);
+				break;
+			case DESPECKLE_NORMAL:
+				despeckleNormalBtn->setChecked(true);
+				break;
+			case DESPECKLE_AGGRESSIVE:
+				despeckleAggressiveBtn->setChecked(true);
+				break;
+		}
+
 		ScopedIncDec<int> const guard(m_ignoreThresholdChanges);
 		thresholdSlider->setValue(
 			m_colorParams.blackWhiteOptions().thresholdAdjustment()
