@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,12 +24,15 @@
 #include "DebugImages.h"
 #include "EstimateBackground.h"
 #include "Despeckle.h"
+#include "Undistort.h"
 #include "RenderParams.h"
 #include "Dpi.h"
 #include "Dpm.h"
 #include "Zone.h"
 #include "ZoneSet.h"
 #include "PictureLayerProperty.h"
+#include "FillColorProperty.h"
+#include "imageproc/GrayImage.h"
 #include "imageproc/BinaryImage.h"
 #include "imageproc/BinaryThreshold.h"
 #include "imageproc/Binarize.h"
@@ -49,6 +52,7 @@
 #include "imageproc/DrawOver.h"
 #include "imageproc/AdjustBrightness.h"
 #include "imageproc/PolygonRasterizer.h"
+#include "config.h"
 #include <boost/foreach.hpp>
 #include <QImage>
 #include <QSize>
@@ -57,8 +61,13 @@
 #include <QRectF>
 #include <QPointF>
 #include <QPolygonF>
+#include <QPainter>
+#include <QColor>
+#include <QPen>
+#include <QBrush>
 #include <QtGlobal>
 #include <QDebug>
+#include <Qt>
 #include <vector>
 #include <algorithm>
 #include <assert.h>
@@ -70,15 +79,144 @@ using namespace imageproc;
 namespace output
 {
 
+namespace
+{
+
+struct RaiseAboveBackground
+{
+	static uint8_t transform(uint8_t src, uint8_t dst) {
+		// src: orig
+		// dst: background (dst >= src)
+		if (dst - src < 1) {
+			return 0xff;
+		}
+		unsigned const orig = src;
+		unsigned const background = dst;
+		return static_cast<uint8_t>((orig * 255 + background / 2) / background);
+	}
+};
+
+struct CombineInverted
+{
+	static uint8_t transform(uint8_t src, uint8_t dst) {
+		unsigned const dilated = dst;
+		unsigned const eroded = src;
+		unsigned const res = 255 - (255 - dilated) * eroded / 255;
+		return static_cast<uint8_t>(res);
+	}
+};
+
+/**
+ * In picture areas we make sure we don't use pure black and pure white colors.
+ * These are reserved for text areas.  This behaviour makes it possible to
+ * detect those picture areas later and treat them differently, for example
+ * encoding them as a background layer in DjVu format.
+ */
+template<typename PixelType>
+PixelType reserveBlackAndWhite(PixelType color);
+
+template<>
+uint32_t reserveBlackAndWhite(uint32_t color)
+{
+	// We handle both RGB32 and ARGB32 here.
+	switch (color & 0x00FFFFFF) {
+		case 0x00000000:
+			return 0xFF010101;
+		case 0x00FFFFFF:
+			return 0xFFFEFEFE;
+		default:
+			return color;
+	}
+}
+
+template<>
+uint8_t reserveBlackAndWhite(uint8_t color)
+{
+	switch (color) {
+		case 0x00:
+			return 0x01;
+		case 0xFF:
+			return 0xFE;
+		default:
+			return color;
+	}
+}
+
+template<typename PixelType>
+void reserveBlackAndWhite(QSize size, int stride, PixelType* data)
+{
+	int const width = size.width();
+	int const height = size.height();
+
+	PixelType* line = data;
+	for (int y = 0; y < height; ++y, line += stride) {
+		for (int x = 0; x < width; ++x) {
+			line[x] = reserveBlackAndWhite<PixelType>(line[x]);
+		}
+	}
+}
+
+/**
+ * Fills areas of \p mixed with pixels from \p bw_content in
+ * areas where \p bw_mask is black.  Supported \p mixed image formats
+ * are Indexed8 grayscale, RGB32 and ARGB32.
+ * The \p MixedPixel type is uint8_t for Indexed8 grayscale and uint32_t
+ * for RGB32 and ARGB32.
+ */
+template<typename MixedPixel>
+void combineMixed(
+	QImage& mixed, BinaryImage const& bw_content,
+	BinaryImage const& bw_mask)
+{
+	MixedPixel* mixed_line = reinterpret_cast<MixedPixel*>(mixed.bits());
+	int const mixed_stride = mixed.bytesPerLine() / sizeof(MixedPixel);
+	uint32_t const* bw_content_line = bw_content.data();
+	int const bw_content_stride = bw_content.wordsPerLine();
+	uint32_t const* bw_mask_line = bw_mask.data();
+	int const bw_mask_stride = bw_mask.wordsPerLine();
+	int const width = mixed.width();
+	int const height = mixed.height();
+	uint32_t const msb = uint32_t(1) << 31;
+	
+	for (int y = 0; y < height; ++y) {
+		for (int x = 0; x < width; ++x) {
+			if (bw_mask_line[x >> 5] & (msb >> (x & 31))) {
+				// B/W content.
+
+				uint32_t tmp = bw_content_line[x >> 5];
+				tmp >>= (31 - (x & 31));
+				tmp &= uint32_t(1);
+				// Now it's 0 for white and 1 for black.
+				
+				--tmp; // 0 becomes 0xffffffff and 1 becomes 0.
+				
+				tmp |= 0xff000000; // Force opacity.
+				
+				mixed_line[x] = static_cast<MixedPixel>(tmp);
+			} else {
+				// Non-B/W content.
+				mixed_line[x] = reserveBlackAndWhite<MixedPixel>(mixed_line[x]);
+			}
+		}
+		mixed_line += mixed_stride;
+		bw_content_line += bw_content_stride;
+		bw_mask_line += bw_mask_stride;
+	}
+}
+
+} // anonymous namespace
+
 OutputGenerator::OutputGenerator(
 	Dpi const& dpi, ColorParams const& color_params,
+	DespeckleLevel const despeckle_level,
 	ImageTransformation const& pre_xform,
 	QPolygonF const& content_rect_phys,
 	QPolygonF const& page_rect_phys)
 :	m_dpi(dpi),
 	m_colorParams(color_params),
 	m_pageRectPhys(page_rect_phys),
-	m_toUncropped(pre_xform.transform())
+	m_toUncropped(pre_xform.transform()),
+	m_despeckleLevel(despeckle_level)
 {
 	QTransform const post_scale(
 		Utils::scaleFromToDpi(pre_xform.preScaledDpi(), m_dpi)
@@ -116,10 +254,18 @@ OutputGenerator::toOutput() const
 
 QImage
 OutputGenerator::process(
-	TaskStatus const& status, FilterData const& input, ZoneSet const& zones,
-	imageproc::BinaryImage* auto_picture_mask, DebugImages* const dbg) const
+	TaskStatus const& status, FilterData const& input,
+	ZoneSet const& picture_zones, ZoneSet const& fill_zones,
+	imageproc::BinaryImage* auto_picture_mask,
+	imageproc::BinaryImage* speckles_image,
+	DebugImages* const dbg) const
 {
-	QImage image(processImpl(status, input, zones, auto_picture_mask, dbg));
+	QImage image(
+		processImpl(
+			status, input, picture_zones, fill_zones,
+			auto_picture_mask, speckles_image, dbg
+		)
+	);
 	assert(!image.isNull());
 	
 	// Set the correct DPI.
@@ -146,12 +292,16 @@ QImage
 OutputGenerator::processAsIs(FilterData const& input,
 	TaskStatus const& status, DebugImages* const dbg) const
 {
-	uint8_t const dominant_gray = calcDominantBackgroundGrayLevel(input.grayImage());
+	uint8_t const dominant_gray = reserveBlackAndWhite<uint8_t>(
+		calcDominantBackgroundGrayLevel(input.grayImage())
+	);
 	
 	status.throwIfCancelled();
 	
 	QColor const bg_color(dominant_gray, dominant_gray, dominant_gray);
 	
+	QImage out;
+
 	if (input.origImage().allGray()) {
 		if (m_cropRect.isEmpty()) {
 			QImage image(1, 1, QImage::Format_Indexed8);
@@ -159,7 +309,8 @@ OutputGenerator::processAsIs(FilterData const& input,
 			image.fill(dominant_gray);
 			return image;
 		}
-		return transformToGray(
+
+		out = transformToGray(
 			input.grayImage(), m_toUncropped, m_cropRect, bg_color
 		);
 	} else {
@@ -168,91 +319,32 @@ OutputGenerator::processAsIs(FilterData const& input,
 			image.fill(bg_color.rgb());
 			return image;
 		}
-		return transform(
-			input.origImage(), m_toUncropped, m_cropRect, bg_color
-		);
+
+		out = transform(input.origImage(), m_toUncropped, m_cropRect, bg_color);
 	}
+
+	assert(out.depth() == 8 || out.depth() == 24 || out.depth() == 32);
+	switch (out.format()) {
+		case QImage::Format_Indexed8:
+			reserveBlackAndWhite(out.size(), out.bytesPerLine(), out.bits());
+			break;
+		case QImage::Format_RGB32:
+		case QImage::Format_ARGB32:
+			reserveBlackAndWhite(out.size(), out.bytesPerLine()/4, (uint32_t*)out.bits());
+			break;
+		default:; // Should not happen.
+	}
+
+	return out;
 }
 
-namespace
-{
-
-struct RaiseAboveBackground
-{
-	static uint8_t transform(uint8_t src, uint8_t dst) {
-		// src: orig
-		// dst: background (dst >= src)
-		if (dst - src < 1) {
-			return 0xff;
-		}
-		unsigned const orig = src;
-		unsigned const background = dst;
-		return static_cast<uint8_t>((orig * 255 + background / 2) / background);
-	}
-};
-
-struct CombineInverted
-{
-	static uint8_t transform(uint8_t src, uint8_t dst) {
-		unsigned const dilated = dst;
-		unsigned const eroded = src;
-		unsigned const res = 255 - (255 - dilated) * eroded / 255;
-		return static_cast<uint8_t>(res);
-	}
-};
-
-/**
- * Fills areas of \p mixed with pixels from \p bw_content in
- * areas where \p bw_mask is black.  Supported \p mixed image formats
- * are Indexed8 grayscale, RGB32 and ARGB32.
- * The \p MixedPixel type is uint8_t for Indexed8 grayscale and uint32_t
- * for RGB32 and ARGB32.
- */
-template<typename MixedPixel>
-void combineMixed(
-	QImage& mixed, BinaryImage const& bw_content,
-	BinaryImage const& bw_mask)
-{
-	MixedPixel* mixed_line = reinterpret_cast<MixedPixel*>(mixed.bits());
-	int const mixed_stride = mixed.bytesPerLine() / sizeof(MixedPixel);
-	uint32_t const* bw_content_line = bw_content.data();
-	int const bw_content_stride = bw_content.wordsPerLine();
-	uint32_t const* bw_mask_line = bw_mask.data();
-	int const bw_mask_stride = bw_mask.wordsPerLine();
-	int const width = mixed.width();
-	int const height = mixed.height();
-	uint32_t const msb = uint32_t(1) << 31;
-	
-	for (int y = 0; y < height; ++y) {
-		for (int x = 0; x < width; ++x) {
-			if (bw_mask_line[x >> 5] & (msb >> (x & 31))) {
-				uint32_t tmp = bw_content_line[x >> 5];
-				tmp >>= (31 - (x & 31));
-				tmp &= uint32_t(1);
-				// Now it's 0 for white and 1 for black.
-				
-				--tmp; // 0 becomes 0xffffffff and 1 becomes 0.
-				
-				tmp |= 0xff000000; // Force opacity.
-				
-				mixed_line[x] = static_cast<MixedPixel>(tmp);
-			}
-		}
-		mixed_line += mixed_stride;
-		bw_content_line += bw_content_stride;
-		bw_mask_line += bw_mask_stride;
-	}
-}
-
-} // anonymous namespace
-
 QImage
 OutputGenerator::normalizeIlluminationGray(
 	TaskStatus const& status,
 	QImage const& input, QPolygonF const& area_to_consider,
 	QTransform const& xform, QRect const& target_rect, DebugImages* const dbg)
 {
-	QImage to_be_normalized(
+	GrayImage to_be_normalized(
 		transformToGray(
 			input, xform, target_rect,
 			Qt::black // <-- Important!
@@ -276,7 +368,7 @@ OutputGenerator::normalizeIlluminationGray(
 	
 	status.throwIfCancelled();
 	
-	QImage background(bg_ps.render(to_be_normalized.size()));
+	GrayImage background(bg_ps.render(to_be_normalized.size()));
 	if (dbg) {
 		dbg->add(background, "background");
 	}
@@ -293,11 +385,10 @@ OutputGenerator::normalizeIlluminationGray(
 
 imageproc::BinaryImage
 OutputGenerator::estimateBinarizationMask(
-	TaskStatus const& status, QImage const& gray_source,
+	TaskStatus const& status, GrayImage const& gray_source,
 	QRect const& source_rect, QRect const& source_sub_rect,
 	DebugImages* const dbg) const
 {
-	assert(gray_source.format() == QImage::Format_Indexed8);
 	assert(source_rect.contains(source_sub_rect));
 	
 	// If we need to strip some of the margins from a grayscale
@@ -306,7 +397,7 @@ OutputGenerator::estimateBinarizationMask(
 	// That image won't own that data, but gray_source is not
 	// going anywhere, so it's fine.
 	
-	QImage trimmed_image;
+	GrayImage trimmed_image;
 	
 	if (source_rect == source_sub_rect) {
 		trimmed_image = gray_source; // Shallow copy.
@@ -317,16 +408,15 @@ OutputGenerator::estimateBinarizationMask(
 			source_sub_rect.topLeft() - source_rect.topLeft()
 		);
 		
-		int const bpl = gray_source.bytesPerLine();
-		int const offset = relative_subrect.top() * bpl
+		int const stride = gray_source.stride();
+		int const offset = relative_subrect.top() * stride
 				+ relative_subrect.left();
 		
-		trimmed_image = QImage(
-			gray_source.bits() + offset,
+		trimmed_image = GrayImage(QImage(
+			gray_source.data() + offset,
 			relative_subrect.width(), relative_subrect.height(),
-			bpl, gray_source.format()
-		);
-		trimmed_image.setColorTable(gray_source.colorTable());
+			stride, QImage::Format_Indexed8
+		));
 	}
 	
 	status.throwIfCancelled();
@@ -334,18 +424,16 @@ OutputGenerator::estimateBinarizationMask(
 	QSize const downscaled_size(to300dpi(trimmed_image.size(), m_dpi));
 	
 	// A 300dpi version of trimmed_image.
-	QImage downscaled_input(
+	GrayImage downscaled_input(
 		scaleToGray(trimmed_image, downscaled_size)
 	);
-	trimmed_image = QImage(); // Save memory.
+	trimmed_image = GrayImage(); // Save memory.
 	
 	status.throwIfCancelled();
 	
 	// Light areas indicate pictures.
-	QImage picture_areas(
-		detectPictures(downscaled_input, status, dbg)
-	);
-	downscaled_input = QImage(); // Save memory.
+	GrayImage picture_areas(detectPictures(downscaled_input, status, dbg));
+	downscaled_input = GrayImage(); // Save memory.
 	
 	status.throwIfCancelled();
 	
@@ -397,13 +485,18 @@ OutputGenerator::modifyBinarizationMask(
 
 QImage
 OutputGenerator::processImpl(
-	TaskStatus const& status, FilterData const& input, ZoneSet const& zones,
-	imageproc::BinaryImage* auto_picture_mask, DebugImages* const dbg) const
+	TaskStatus const& status, FilterData const& input,
+	ZoneSet const& picture_zones, ZoneSet const& fill_zones,
+	imageproc::BinaryImage* auto_picture_mask,
+	imageproc::BinaryImage* speckles_image,
+	DebugImages* const dbg) const
 {
 	RenderParams const render_params(m_colorParams);
 	
 	if (!render_params.whiteMargins()) {
-		return processAsIs(input, status, dbg);
+		QImage out(processAsIs(input, status, dbg));
+		applyFillZonesInPlace(out, fill_zones);
+		return out;
 	}
 	
 	// The whole image minus the part cut off by the split line.
@@ -411,7 +504,7 @@ OutputGenerator::processImpl(
 	
 	// For various reasons, we need some whitespace around the content
 	// area.  This is the number of pixels of such whitespace.
-	int const content_margin = 20;
+	int const content_margin = m_dpi.vertical() * 20 / 300;
 	
 	// The content area (in m_toUncropped coordinates) extended
 	// with content_margin.  Note that we prevent that extension
@@ -462,7 +555,7 @@ OutputGenerator::processImpl(
 			normalize_illumination_rect, Qt::white
 		);
 	}
-	
+
 	status.throwIfCancelled();
 	
 	QImage maybe_smoothed;
@@ -479,7 +572,7 @@ OutputGenerator::processImpl(
 	
 	status.throwIfCancelled();
 	
-	if (render_params.binaryOutput()) {
+	if (render_params.binaryOutput() || m_cropRect.isEmpty()) {
 		BinaryImage dst(m_cropRect.size().expandedTo(QSize(1, 1)), WHITE);
 		
 		if (!m_contentRect.isEmpty()) {
@@ -492,33 +585,40 @@ OutputGenerator::processImpl(
 			
 			status.throwIfCancelled();
 			
-			if (render_params.despeckle()) {
-				despeckleInPlace(bw_content, m_dpi, status, dbg);
-				if (dbg) {
-					dbg->add(bw_content, "despeckled");
-				}
-			}
-			
-			status.throwIfCancelled();
-			
 			morphologicalSmoothInPlace(bw_content, status);
 			if (dbg) {
 				dbg->add(bw_content, "edges_smoothed");
 			}
-			
+
 			status.throwIfCancelled();
 			
-			QRect dst_rect(m_contentRect);
-			dst_rect.moveTopLeft(m_contentRect.topLeft() - m_cropRect.topLeft());
+			QRect const src_rect(m_contentRect.translated(-normalize_illumination_rect.topLeft()));
+			QRect const dst_rect(m_contentRect.translated(-m_cropRect.topLeft()));
+			rasterOp<RopSrc>(dst, dst_rect, bw_content, src_rect.topLeft());
+			bw_content.release(); // Save memory.
+
+#if defined(ENABLE_DEWARPING)
+			if (render_params.dewarp()) {
+				dst = undistort(dst, status, dbg);
+				if (dbg) {
+					dbg->add(dst, "undistorted");
+				}
+			}
+
+			status.throwIfCancelled();
+#endif
 			
-			QPoint const src_pos(
-				m_contentRect.topLeft() -
-				normalize_illumination_rect.topLeft()
+			// It's important to keep despeckling the very last operation
+			// affecting the binary part of the output. That's because
+			// we will be reconstructing the input to this despeckling
+			// operation from the final output file.
+			maybeDespeckleInPlace(
+				dst, m_cropRect, m_cropRect, m_despeckleLevel,
+				speckles_image, m_dpi, status, dbg
 			);
-			
-			rasterOp<RopSrc>(dst, dst_rect, bw_content, src_pos);
 		}
 		
+		applyFillZonesInPlace(dst, fill_zones);
 		return dst.toQImage();
 	}
 	
@@ -531,7 +631,7 @@ OutputGenerator::processImpl(
 		// maybe_normalized from grayscale to color mode.
 		
 		bw_mask = estimateBinarizationMask(
-			status, toGrayscale(maybe_normalized),
+			status, GrayImage(maybe_normalized),
 			normalize_illumination_rect,
 			small_margins_rect, dbg
 		);
@@ -546,15 +646,15 @@ OutputGenerator::processImpl(
 			auto_picture_mask->fill(BLACK);
 
 			if (!m_contentRect.isEmpty()) {
+				QRect const src_rect(m_contentRect.translated(-small_margins_rect.topLeft()));
 				QRect const dst_rect(m_contentRect.translated(-m_cropRect.topLeft()));
-				QPoint const src_pos(m_contentRect.topLeft() - small_margins_rect.topLeft());
-				rasterOp<RopSrc>(*auto_picture_mask, dst_rect, bw_mask, src_pos);
+				rasterOp<RopSrc>(*auto_picture_mask, dst_rect, bw_mask, src_rect.topLeft());
 			}
 		}
 
 		status.throwIfCancelled();
 
-		modifyBinarizationMask(bw_mask, small_margins_rect, zones);
+		modifyBinarizationMask(bw_mask, small_margins_rect, picture_zones);
 		if (dbg) {
 			dbg->add(bw_mask, "bw_mask with zones");
 		}
@@ -591,19 +691,27 @@ OutputGenerator::processImpl(
 		
 		status.throwIfCancelled();
 		
-		if (render_params.despeckle()) {
-			despeckleInPlace(bw_content, m_dpi, status, dbg);
-			if (dbg) {
-				dbg->add(bw_content, "despeckled");
-			}
-		}
-		
-		status.throwIfCancelled();
-		
 		morphologicalSmoothInPlace(bw_content, status);
 		if (dbg) {
 			dbg->add(bw_content, "edges_smoothed");
 		}
+
+		status.throwIfCancelled();
+		
+		// We don't want speckles in non-B/W areas, as they would
+		// then get visualized on the Despeckling tab.
+		rasterOp<RopAnd<RopSrc, RopDst> >(bw_content, bw_mask);
+
+		status.throwIfCancelled();
+
+		// It's important to keep despeckling the very last operation
+		// affecting the binary part of the output. That's because
+		// we will be reconstructing the input to this despeckling
+		// operation from the final output file.
+		maybeDespeckleInPlace(
+			bw_content, small_margins_rect, m_contentRect,
+			m_despeckleLevel, speckles_image, m_dpi, status, dbg
+		);
 		
 		status.throwIfCancelled();
 		
@@ -632,15 +740,12 @@ OutputGenerator::processImpl(
 	}
 
 	if (!m_contentRect.isEmpty()) {
-		QRect src_rect(m_contentRect);
-		src_rect.moveTopLeft(
-			m_contentRect.topLeft() - small_margins_rect.topLeft()
-		);
-		
+		QRect const src_rect(m_contentRect.translated(-small_margins_rect.topLeft()));
 		QRect const dst_rect(m_contentRect.translated(-m_cropRect.topLeft()));
 		drawOver(dst, dst_rect, maybe_normalized, src_rect);
 	}
 	
+	applyFillZonesInPlace(dst, fill_zones);
 	return dst;
 }
 
@@ -664,9 +769,9 @@ OutputGenerator::to300dpi(QSize const& size, Dpi const& source_dpi)
 	return QSize(std::max(1, width), std::max(1, height));
 }
 
-QImage
+GrayImage
 OutputGenerator::detectPictures(
-	QImage const& input_300dpi, TaskStatus const& status,
+	GrayImage const& input_300dpi, TaskStatus const& status,
 	DebugImages* const dbg)
 {
 	// We stretch the range of gray levels to cover the whole
@@ -674,40 +779,40 @@ OutputGenerator::detectPictures(
 	// and background to be equally far from the center
 	// of the whole range.  Otherwise text printed with a big
 	// font will be considered a picture.
-	QImage stretched(stretchGrayRange(input_300dpi, 0.01, 0.01));
+	GrayImage stretched(stretchGrayRange(input_300dpi, 0.01, 0.01));
 	if (dbg) {
 		dbg->add(stretched, "stretched");
 	}
 	
 	status.throwIfCancelled();
 	
-	QImage eroded(erodeGray(stretched, QSize(3, 3), 0x00));
+	GrayImage eroded(erodeGray(stretched, QSize(3, 3), 0x00));
 	if (dbg) {
 		dbg->add(eroded, "eroded");
 	}
 	
 	status.throwIfCancelled();
 	
-	QImage dilated(dilateGray(stretched, QSize(3, 3), 0xff));
+	GrayImage dilated(dilateGray(stretched, QSize(3, 3), 0xff));
 	if (dbg) {
 		dbg->add(dilated, "dilated");
 	}
 	
-	stretched = QImage(); // Save memory.
+	stretched = GrayImage(); // Save memory.
 	
 	status.throwIfCancelled();
 	
 	grayRasterOp<CombineInverted>(dilated, eroded);
-	QImage gray_gradient(dilated);
-	dilated = QImage();
-	eroded = QImage();
+	GrayImage gray_gradient(dilated);
+	dilated = GrayImage();
+	eroded = GrayImage();
 	if (dbg) {
 		dbg->add(gray_gradient, "gray_gradient");
 	}
 	
 	status.throwIfCancelled();
 	
-	QImage marker(erodeGray(gray_gradient, QSize(35, 35), 0x00));
+	GrayImage marker(erodeGray(gray_gradient, QSize(35, 35), 0x00));
 	if (dbg) {
 		dbg->add(marker, "marker");
 	}
@@ -715,8 +820,8 @@ OutputGenerator::detectPictures(
 	status.throwIfCancelled();
 	
 	seedFillGrayInPlace(marker, gray_gradient, CONN8);
-	QImage reconstructed(marker);
-	marker = QImage();
+	GrayImage reconstructed(marker);
+	marker = GrayImage();
 	if (dbg) {
 		dbg->add(reconstructed, "reconstructed");
 	}
@@ -730,9 +835,9 @@ OutputGenerator::detectPictures(
 	
 	status.throwIfCancelled();
 	
-	QImage holes_filled(createFramedImage(reconstructed.size()));
+	GrayImage holes_filled(createFramedImage(reconstructed.size()));
 	seedFillGrayInPlace(holes_filled, reconstructed, CONN8);
-	reconstructed = QImage();
+	reconstructed = GrayImage();
 	if (dbg) {
 		dbg->add(holes_filled, "holes_filled");
 	}
@@ -767,7 +872,10 @@ OutputGenerator::adjustThreshold(BinaryThreshold threshold) const
 {
 	int const adjusted = threshold +
 		m_colorParams.blackWhiteOptions().thresholdAdjustment();
-	return BinaryThreshold(qBound(0, adjusted, 255));
+
+	// Hard-bounding threshold values is necessary for example
+	// if all the content went into the picture mask.
+	return BinaryThreshold(qBound(30, adjusted, 225));
 }
 
 BinaryImage
@@ -807,22 +915,76 @@ OutputGenerator::binarize(QImage const& image,
 }
 
 /**
- * Remove small connected components that are considered to be garbage.
+ * \brief Remove small connected components that are considered to be garbage.
+ *
  * Both the size and the distance to other components are taken into account.
+ *
+ * \param[in,out] image The image to despeckle.
+ * \param image_rect The rectangle corresponding to \p image in the same
+ *        coordinate system where m_contentRect and m_cropRect are defined.
+ * \param mask_rect The area within the image to consider.  Defined not
+ *        relative to \p image, but in the same coordinate system where
+ *        m_contentRect and m_cropRect are defined.  This only affects
+ *        \p speckles_img, if provided.
+ * \param level Despeckling aggressiveness.
+ * \param speckles_img If provided, the removed black speckles will be written
+ *        there.  The speckles image is always considered to correspond
+ *        to m_cropRect, so it will have the size of m_cropRect.size().
+ *        Only the area within \p mask_rect will be copied to \p speckles_img.
+ *        The rest will be filled with white.
+ * \param dpi The DPI of the input image.  See the note below.
+ * \param status Task status.
+ * \param dbg An optional sink for debugging images.
+ *
  * \note This function only works effectively when the DPI is symmetric,
  * that is, its horizontal and vertical components are equal.
  */
 void
-OutputGenerator::despeckleInPlace(
-	imageproc::BinaryImage& image, Dpi const& dpi,
-	TaskStatus const& status, DebugImages* dbg)
+OutputGenerator::maybeDespeckleInPlace(
+	imageproc::BinaryImage& image,
+	QRect const& image_rect, QRect const& mask_rect,
+	DespeckleLevel const level, BinaryImage* speckles_img,
+	Dpi const& dpi, TaskStatus const& status, DebugImages* dbg) const
 {
-	int const min_dpi = std::min(dpi.horizontal(), dpi.vertical());
-	double const factor = min_dpi / 300.0;
-	double const squared_factor = factor * factor;
-	
-	int const big_object_threshold = qRound(100 * squared_factor);
-	::despeckleInPlace(image, big_object_threshold, dbg);
+	QRect const src_rect(mask_rect.translated(-image_rect.topLeft()));
+	QRect const dst_rect(mask_rect.translated(-m_cropRect.topLeft()));
+
+	if (speckles_img) {
+		BinaryImage(m_cropRect.size(), WHITE).swap(*speckles_img);
+		if (!mask_rect.isEmpty()) {
+			rasterOp<RopSrc>(*speckles_img, dst_rect, image, src_rect.topLeft());
+		}
+	}
+
+	if (level != DESPECKLE_OFF) {
+		Despeckle::Level lvl = Despeckle::NORMAL;
+		switch (level) {
+			case DESPECKLE_CAUTIOUS:
+				lvl = Despeckle::CAUTIOUS;
+				break;
+			case DESPECKLE_NORMAL:
+				lvl = Despeckle::NORMAL;
+				break;
+			case DESPECKLE_AGGRESSIVE:
+				lvl = Despeckle::AGGRESSIVE;
+				break;
+			default:;
+		}
+
+		Despeckle::despeckleInPlace(image, dpi, lvl, status, dbg);
+
+		if (dbg) {
+			dbg->add(image, "despeckled");
+		}
+	}
+
+	if (speckles_img) {
+		if (!mask_rect.isEmpty()) {
+			rasterOp<RopSubtract<RopDst, RopSrc> >(
+				*speckles_img, dst_rect, image, src_rect.topLeft()
+			);
+		}
+	}
 }
 
 void
@@ -979,19 +1141,6 @@ OutputGenerator::calcLocalWindowSize(Dpi const& dpi)
 	return size_pixels;
 }
 
-void
-OutputGenerator::colorizeBitonal(
-	QImage& img, QRgb const light_color, QRgb const dark_color)
-{
-	if (qGray(img.color(0)) < qGray(img.color(1))) {
-		img.setColor(0, dark_color);
-		img.setColor(1, light_color);
-	} else {
-		img.setColor(0, light_color);
-		img.setColor(1, dark_color);
-	}
-}
-
 unsigned char
 OutputGenerator::calcDominantBackgroundGrayLevel(QImage const& img)
 {
@@ -1033,4 +1182,53 @@ OutputGenerator::calcDominantBackgroundGrayLevel(QImage const& img)
 	return 0;
 }
 
+void
+OutputGenerator::applyFillZonesInPlace(
+	QImage& img, ZoneSet const& zones) const
+{
+	if (zones.empty()) {
+		return;
+	}
+
+	QTransform const xform(toOutput());
+	QImage canvas(img.convertToFormat(QImage::Format_ARGB32_Premultiplied));
+	
+	{
+		QPainter painter(&canvas);
+		painter.setRenderHint(QPainter::Antialiasing, true);
+		painter.setPen(Qt::NoPen);
+
+		BOOST_FOREACH(Zone const& zone, zones) {
+			QColor const color(zone.properties().locateOrDefault<FillColorProperty>()->color());
+			QPolygonF const poly(zone.spline().transformed(xform).toPolygon());
+			painter.setBrush(color);
+			painter.drawPolygon(poly, Qt::WindingFill);
+		}
+	}
+
+	if (img.format() == QImage::Format_Indexed8 && img.isGrayscale()) {
+		img = toGrayscale(canvas);
+	} else {
+		img = canvas.convertToFormat(img.format());
+	}
+}
+
+void
+OutputGenerator::applyFillZonesInPlace(
+	imageproc::BinaryImage& img, ZoneSet const& zones) const
+{
+	if (zones.empty()) {
+		return;
+	}
+
+	QTransform const xform(toOutput());
+
+	BOOST_FOREACH(Zone const& zone, zones) {
+		QColor const color(zone.properties().locateOrDefault<FillColorProperty>()->color());
+		BWColor const bw_color = qGray(color.rgb()) < 128 ? BLACK : WHITE;
+		QPolygonF const poly(zone.spline().transformed(xform).toPolygon());
+		PolygonRasterizer::fill(img, bw_color, poly, Qt::WindingFill);
+	}
+}
+
 } // namespace output
