@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -58,6 +58,8 @@ public:
 		PageId const& page_id, IntrusivePtr<Settings> const& settings);
 	
 	virtual ~PictureZoneEditor();
+signals:
+	void invalidateThumbnail(PageId const& page_id);
 protected:
 	virtual void onPaint(QPainter& painter, InteractionState const& interaction);
 private slots:
@@ -66,6 +68,8 @@ private slots:
 	void initiateBuildingScreenPictureMask();
 
 	void commitZones();
+
+	void updateRequested();
 private:
 	class MaskTransformTask;
 	
