@@ -50,15 +50,15 @@ public:
 				== NEED_BINARIZATION;
 	}
 	
-	bool despeckle() const {
-		return (m_mask & DESPECKLE) != 0;
+	bool dewarp() const {
+		return (m_mask & DEWARP) != 0;
 	}
 private:
 	enum {
 		WHITE_MARGINS = 1,
 		NORMALIZE_ILLUMINATION = 2,
 		NEED_BINARIZATION = 4,
-		DESPECKLE = 8,
+		DEWARP = 8,
 		MIXED_OUTPUT = 16
 	};
 	
