@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,7 +23,8 @@
 #include "RefCountable.h"
 #include "FilterResult.h"
 #include "PageId.h"
-#include <QString>
+#include "ImageViewTab.h"
+#include "OutputFileNameGenerator.h"
 #include <QColor>
 #include <memory>
 
@@ -54,9 +55,9 @@ class Task : public RefCountable
 public:
 	Task(IntrusivePtr<Filter> const& filter,
 		IntrusivePtr<Settings> const& settings,
-		ThumbnailPixmapCache& thumbnail_cache,
-		PageId const& page_id, int page_num,
-		QString const& out_dir, bool batch, bool debug);
+		IntrusivePtr<ThumbnailPixmapCache> const& thumbnail_cache,
+		PageId const& page_id, OutputFileNameGenerator const& out_file_name_gen,
+		ImageViewTab last_tab, bool batch, bool debug);
 	
 	virtual ~Task();
 	
@@ -67,14 +68,17 @@ public:
 private:
 	class UiUpdater;
 	
+	void deleteMutuallyExclusiveOutputFiles();
+
 	IntrusivePtr<Filter> m_ptrFilter;
 	IntrusivePtr<Settings> m_ptrSettings;
-	ThumbnailPixmapCache& m_rThumbnailCache;
+	IntrusivePtr<ThumbnailPixmapCache> m_ptrThumbnailCache;
 	std::auto_ptr<DebugImages> m_ptrDbg;
 	PageId m_pageId;
-	QString m_outDir;
-	int m_pageNum;
+	OutputFileNameGenerator m_outFileNameGen;
+	ImageViewTab m_lastTab;
 	bool m_batchProcessing;
+	bool m_debug;
 };
 
 } // namespace output
