@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,9 +38,10 @@ namespace page_layout
 {
 
 Thumbnail::Thumbnail(
-	ThumbnailPixmapCache& thumbnail_cache, QSizeF const& max_size,
-	ImageId const& image_id, ImageTransformation const& xform,
-	Params const& params, QRectF const& adapted_content_rect,
+	IntrusivePtr<ThumbnailPixmapCache> const& thumbnail_cache,
+	QSizeF const& max_size, ImageId const& image_id,
+	ImageTransformation const& xform, Params const& params,
+	QRectF const& adapted_content_rect,
 	QSizeF const& aggregate_hard_size_mm)
 :	ThumbnailBase(thumbnail_cache, max_size, image_id, xform),
 	m_params(params),
@@ -63,8 +64,11 @@ Thumbnail::paintOverImage(
 	QTransform const orig_to_presentation(
 		m_origXform.transformBack() * imageXform().transform()
 	);
+	QTransform const presentation_to_thumb(
+		image_to_display * thumb_to_display.inverted()
+	);
 	QTransform const orig_to_thumb(
-		orig_to_presentation * image_to_display * thumb_to_display.inverted()
+		orig_to_presentation * presentation_to_thumb
 	);
 	
 	// We work in thumbnail coordinates because we want to adjust
@@ -87,23 +91,40 @@ Thumbnail::paintOverImage(
 	QPainterPath content_outline;
 	content_outline.addPolygon(PolygonUtils::round(inner_rect));
 	
+	QPolygonF const orig_image_outline(
+		m_origXform.transform().map(m_origXform.origRect())
+	);
+
 	QPainterPath page_outline;
 	page_outline.addPolygon(
 		PolygonUtils::round(
-			orig_to_thumb.map(m_origXform.resultingCropArea())
+			orig_to_thumb.map(orig_image_outline.intersected(m_outerRect))
 		)
 	);
 	
-	painter.setRenderHint(QPainter::Antialiasing, false);
+	painter.setRenderHint(QPainter::Antialiasing, true);
 	
 	// Clear parts of the thumbnail that don't belong to the image
 	// but belong to outer_rect.
 	painter.fillPath(outer_outline.subtracted(page_outline), QApplication::palette().window());
 	
+	QColor bg_color;
+	QColor fg_color;
+	if (m_params.alignment().isNull()) {
+		// "Align with other pages" is turned off.
+		// Different color is useful on a thumbnail list to
+		// distinguish "safe" pages from potentially problematic ones.
+		bg_color = QColor(0x58, 0x7f, 0xf4, 70);
+		fg_color = QColor(0x00, 0x52, 0xff);
+	} else {
+		bg_color = QColor(0xbb, 0x00, 0xff, 40);
+		fg_color = QColor(0xbe, 0x5b, 0xec);
+	}
+
 	// Draw margins.
-	painter.fillPath(outer_outline.subtracted(content_outline), QColor(0xbb, 0x00, 0xff, 40));
+	painter.fillPath(outer_outline.subtracted(content_outline), bg_color);
 	
-	QPen pen(QColor(0xbe, 0x5b, 0xec));
+	QPen pen(fg_color);
 	pen.setCosmetic(true);
 	pen.setWidthF(1.0);
 	painter.setPen(pen);
