@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,8 +43,7 @@ CacheDrivenTask::~CacheDrivenTask()
 
 void
 CacheDrivenTask::process(
-	PageInfo const& page_info, int const page_num,
-	AbstractFilterDataCollector* collector,
+	PageInfo const& page_info, AbstractFilterDataCollector* collector,
 	ImageTransformation const& xform)
 {
 	Settings::Record const record(
@@ -75,10 +74,16 @@ CacheDrivenTask::process(
 		return;
 	}
 	
-	PageLayout const layout(params->pageLayout());
-	
+	PageLayout layout(params->pageLayout());
+	if (layout.uncutOutline().isEmpty()) {
+		// Backwards compatibility with versions < 0.9.9
+		layout.setUncutOutline(xform.resultingRect());
+	}
+
 	if (m_ptrNextTask) {
-		m_ptrNextTask->process(page_info, page_num, collector, xform, layout);
+		ImageTransformation new_xform(xform);
+		new_xform.setCropArea(layout.pageOutline(page_info.id().subPage()));
+		m_ptrNextTask->process(page_info, collector, new_xform);
 		return;
 	}
 	
@@ -88,7 +93,9 @@ CacheDrivenTask::process(
 				new Thumbnail(
 					thumb_col->thumbnailCache(),
 					thumb_col->maxLogicalThumbSize(),
-					page_info.imageId(), xform, layout
+					page_info.imageId(), xform, layout,
+					page_info.leftHalfRemoved(),
+					page_info.rightHalfRemoved()
 				)
 			)
 		);
