@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,8 +45,7 @@ CacheDrivenTask::~CacheDrivenTask()
 
 void
 CacheDrivenTask::process(
-	PageInfo const& page_info, int const page_num,
-	AbstractFilterDataCollector* collector,
+	PageInfo const& page_info, AbstractFilterDataCollector* collector,
 	ImageTransformation const& xform)
 {
 	std::auto_ptr<Params> params(m_ptrSettings->getPageParams(page_info.id()));
@@ -73,9 +72,7 @@ CacheDrivenTask::process(
 	}
 	
 	if (m_ptrNextTask) {
-		m_ptrNextTask->process(
-			page_info, page_num, collector, xform, params->contentRect()
-		);
+		m_ptrNextTask->process(page_info, collector, xform, params->contentRect());
 		return;
 	}
 	
