@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "Dependencies.h"
 #include "AutoManualMode.h"
 #include <QRectF>
+#include <QSizeF>
 
 class QDomDocument;
 class QDomElement;
@@ -35,13 +36,16 @@ class Params
 public:
 	// Member-wise copying is OK.
 	
-	Params(QRectF const& rect, Dependencies const& deps, AutoManualMode mode);
+	Params(QRectF const& rect, QSizeF const& size_mm,
+		Dependencies const& deps, AutoManualMode mode);
 	
 	Params(QDomElement const& filter_el);
 	
 	~Params();
 	
 	QRectF const& contentRect() const { return m_contentRect; }
+
+	QSizeF const& contentSizeMM() const { return m_contentSizeMM; }
 	
 	Dependencies const& dependencies() const { return m_deps; }
 	
@@ -50,6 +54,7 @@ public:
 	QDomElement toXml(QDomDocument& doc, QString const& name) const;
 private:
 	QRectF m_contentRect;
+	QSizeF m_contentSizeMM;
 	Dependencies m_deps;
 	AutoManualMode m_mode;
 };
