@@ -1,6 +1,6 @@
 /*
 	Scan Tailor - Interactive post-processing tool for scanned pages.
-	Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
 	This program is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "XmlUnmarshaller.h"
 #include <QDomDocument>
 #include <QDomElement>
+#include <QTransform>
 #include <QString>
 #include <QPointF>
 #include <boost/foreach.hpp>
@@ -65,3 +66,16 @@ SerializableSpline::toXml(QDomDocument& doc, QString const& name) const
 
 	return el;
 }
+
+SerializableSpline
+SerializableSpline::transformed(QTransform const& xform) const
+{
+	SerializableSpline transformed(*this);
+
+	BOOST_FOREACH(QPointF& pt, transformed.m_points) {
+		pt = xform.map(pt);
+	}
+
+	return transformed;
+}
+
